/**********************************************************************************
MaksLand Home 1997 - 1998
=========================
Module Name:
    regtree.c

Abstract: Regestry operations for a TreeView 
**********************************************************************************/
#include <windows.h>
#include <memory.h>
#include <commctrl.h>

#include "tree.h"
#include "resource.h"
#include "global.h"

#include  "regtree.h"

//-------------------------------------------------------------------------------
typedef struct
{
  HTREEITEM  hTreeItem;
  LPTSTR     pszName;
  HKEY       hKey;
} CRegRootItems;

//-------------------------------------------------------------------------------
static TCHAR sz0 [] = TEXT("My computer");
static TCHAR sz1 [] = TEXT("HKEY_CLASSES_ROOT");
static TCHAR sz2 [] = TEXT("HKEY_CURRENT_USER");
static TCHAR sz3 [] = TEXT("HKEY_LOCAL_MACHINE");
static TCHAR sz4 [] = TEXT("HKEY_USERS");
static TCHAR * pszKeys [] = {sz0, sz1, sz2, sz3, sz4};
static HKEY  hKeys[NUM_ROOT_ITEMS] = {0, HKEY_CLASSES_ROOT, HKEY_CURRENT_USER, 
                                         HKEY_LOCAL_MACHINE, HKEY_USERS};
//-------------------------------------------------------------------------------
CRegRootItems  RootItems[NUM_ROOT_ITEMS];

//-------------------------------------------------------------------------------
static HKEY    isItemRoot (HTREEITEM h_item);
static BOOLEAN isParent (HKEY h_parent, TCHAR * psz_key);

//**********************************************************************************
//  Build complete key from tree's item
//  Input : h_tree - TreeView window
//          item   - tree's item
//  Output: h_key      - root key handle
//          psz_subKey - name(path) of subkey
void  RegTree_BuildKey (HWND h_tree, TV_ITEM item, HKEY * p_hkey, LPTSTR psz_subKey)
{
  HTREEITEM   h_ti;
	TCHAR       sz_getBuff [MAX_PATH+1];
	TCHAR       sz_buff [MAX_PATH*8];

	h_ti = item.hItem;

  *p_hkey = isItemRoot (h_ti);
  if (*p_hkey != NULL)
  {
    psz_subKey[0] = 0;
    return;
  }
  
  // Get the text for the first item
  item.mask = TVIF_TEXT;
  item.pszText = sz_getBuff;
  item.cchTextMax =  MAX_PATH;

  if (!TreeView_GetItem(h_tree, &item))
    return;

  // Create the initial string
  lstrcpy (psz_subKey, item.pszText);

  // Now we move from current item up to the root key that is always open
  // and add the parent directories (if any) to the subkey path
  while (h_ti = TreeView_GetParent(h_tree, h_ti))
  {
    *p_hkey = isItemRoot (h_ti);
    if (*p_hkey != NULL)
      break;

    item.hItem = h_ti;
    item.mask = TVIF_TEXT;
    item.pszText = sz_getBuff;
    item.cchTextMax = MAX_PATH;
    if (!TreeView_GetItem(h_tree, &item))
      return;
     
    // copy parent to the begin of the subkey name
		lstrcpy (sz_buff, psz_subKey); 
		wsprintf (psz_subKey, TEXT("%s\\%s"), item.pszText, sz_buff);
  }
}

//**********************************************************************************
// Load all subkey names for h_rootKey+psz_subKey to the h_tree
void  RegTree_LoadKey  (HWND h_tree, HTREEITEM h_item, HKEY h_rootKey, TCHAR * psz_subKey)
{
  HKEY     h_key;
  BOOLEAN  bool_isSubKeys = 0;
  TCHAR    psz_nextKey [MAX_PATH+1];
  DWORD    n_keyNameSize; 
  DWORD    dw_index;

  DWORD retCode = RegOpenKeyEx (h_rootKey, psz_subKey, 0, KEY_EXECUTE, &h_key);
  if (retCode != ERROR_SUCCESS)
    return;

  for (dw_index = 0 ; ; dw_index ++) //--- Read all sub keys 
  {
    n_keyNameSize = MAX_PATH;
    if (RegEnumKeyEx (h_key, dw_index, psz_nextKey, &n_keyNameSize, 0, NULL, NULL, NULL) != ERROR_SUCCESS)
      break;
    bool_isSubKeys = isParent (h_key, psz_nextKey);
    Tree_AddItem (h_tree, psz_nextKey, h_item, bool_isSubKeys);
  }
  RegCloseKey (h_key);
}


//**********************************************************************************
//  Load all root keys to the h_tree window
HTREEITEM RegTree_LoadRoot (HWND h_tree)
{  
  BOOL  bool_isSubKeys;
  int   i;

  for (i = 0; i < NUM_ROOT_ITEMS; i++)
  {
    if (i != 0)
      bool_isSubKeys = isParent (hKeys[i], NULL); //TEXT(""));
    else
      bool_isSubKeys = TRUE;

    RootItems[i].hTreeItem = Tree_AddItem (h_tree, pszKeys[i],  RootItems[0].hTreeItem, bool_isSubKeys);
    RootItems[i].pszName = pszKeys [i];
    RootItems[i].hKey = hKeys[i];
  }
  return RootItems[0].hTreeItem;
}

//**********************************************************************************
void  RegTree_GetRootPath (HKEY h_rootKey, TCHAR * psz_path)
{
  int i;

  lstrcpy (psz_path, pszKeys[0]);
  lstrcat (psz_path, TEXT("\\"));
  
  if (h_rootKey == NULL)
    return;

  for (i = 1; i < NUM_ROOT_ITEMS; i++)
  {
    if (RootItems[i].hKey == h_rootKey)
    {
      lstrcat (psz_path, RootItems[i].pszName);
      lstrcat (psz_path, TEXT("\\"));
    }
  }
}

//**********************************************************************************
static HKEY isItemRoot (HTREEITEM h_item)
{
  int i;

  for (i=0; i < NUM_ROOT_ITEMS; i++)
  {
    if (RootItems[i].hTreeItem == h_item)
      return RootItems[i].hKey;
  }          
  return NULL;
}

//**********************************************************************************
//   If h_parent+psz_key has subkey(s) -- it is a parent
static BOOLEAN isParent (HKEY h_parent, TCHAR * psz_key)
{
  DWORD  n_numSubKeys; // address of buffer for number of subkeys 
  DWORD  n_maxSubKeyLen;// address of buffer for longest subkey name length  
  DWORD  n_maxClassLen; // address of buffer for longest class string length 
  DWORD  n_numValues; // address of buffer for number of value entries 
  DWORD  n_maxValueNameLen;     // address of buffer for longest value name length 

  HKEY  h_key; // handle of key to query 
  DWORD retCode = RegOpenKeyEx (h_parent, psz_key, 0, KEY_EXECUTE, &h_key);
  if (retCode != ERROR_SUCCESS)
    return FALSE;

  n_numSubKeys = 0;
  retCode = RegQueryInfoKey (h_key, NULL, NULL, NULL, 
                              &n_numSubKeys, &n_maxSubKeyLen, 
                              &n_maxClassLen,
                              &n_numValues, &n_maxValueNameLen, NULL, 
                              NULL, NULL);
  RegCloseKey (h_key);
  return (n_numSubKeys != 0);
}